@@ -1,6 +1,6 @@
 from request_handling import FourOhFourException
 
-    
+
 class AbstractQueryset(object):
     """The design of the `AbstractQueryset` attempts to map RESTful calls
     directly to CRUD calls. It also attempts to be compatible with a single
@@ -26,7 +26,7 @@ class AbstractQueryset(object):
     MSG_CREATED = 'Created'
     MSG_NOTFOUND = 'Not Found'
     MSG_FAILED = 'Failed'
-    
+
     def __init__(self, db_conn=None, api_id='id'):
         self.db_conn = db_conn
         self.api_id = api_id
@@ -78,7 +78,7 @@ class AbstractQueryset(object):
     ###
 
     ### Create Functions
-    
+
     def create_one(self, shield):
         raise NotImplementedError
 
@@ -131,7 +131,7 @@ class DictQueryset(AbstractQueryset):
         super(DictQueryset, self).__init__(db_conn=dict(), **kw)
 
     ### Create Functions
-        
+
     def create_one(self, shield):
         if shield.id in self.db_conn:
             status = self.MSG_UPDATED
@@ -155,18 +155,14 @@ class DictQueryset(AbstractQueryset):
     def read_one(self, iid):
         iid = str(iid)  # TODO Should be cleaner
         if iid in self.db_conn:
-            return (self.MSG_UPDATED, self.db_conn[iid])
+            return (self.MSG_OK, self.db_conn[iid])
         else:
             return (self.MSG_FAILED, iid)
 
     def read_many(self, ids):
-        try:
-            return [self.read_one(iid) for iid in ids]
-        except KeyError:
-            raise FourOhFourException
+        return [self.read_one(iid) for iid in ids]
 
     ### Update Functions
-
     def update_one(self, shield):
         shield_key = str(getattr(shield, self.api_id))
         self.db_conn[shield_key] = shield.to_python()
@@ -180,11 +176,11 @@ class DictQueryset(AbstractQueryset):
 
     def destroy_one(self, item_id):
         try:
-            shield = self.db_conn[item_id]
+            datum = self.db_conn[item_id]
             del self.db_conn[item_id]
         except KeyError:
             raise FourOhFourException
-        return (self.MSG_UPDATED, shield)
+        return (self.MSG_UPDATED, datum)
 
     def destroy_many(self, ids):
         statuses = [self.destroy_one(iid) for iid in ids]
